

from pydantic import BaseModel


class FlockAgentConfig(BaseModel):
    """FlockAgentConfig is a class that holds the configuration for a Flock agent.
    It is used to store various settings and parameters that can be accessed throughout the agent's lifecycle.
    """

    pass
